% AR(1) plus noise model - filtering and real time estimation 
clear all; clc, clf;
cn  = 1000;
%--------------- Parameters 
dphi = 0.9;      % AR parameter
dsnratio = 0.14; 
dsigma2_eps = .2; % Irregular disturbance variance
%--------------- 
vepsilon = sqrt(dsigma2_eps) * randn(cn, 1);
veta = sqrt(dsnratio * dsigma2_eps) * randn(cn, 1);
var1 = filter(1, [1 -dphi], veta);
vy = var1 + vepsilon;
plot(1:cn, vy, '-r', 1:cn, var1, '-b'); 
title('Series and true AR(1) signal');
legend('Series', 'AR(1)');
%--------------- State space - System matrices
mZ = 1; mGG = 1; dsigma2 = dsigma2_eps;
mT = dphi;  mHH = dsnratio; mHG = 0;
va = 0; mP = dsnratio/(1-dphi^2); % initial conditions
%-------------------------------------------------------------------------
[vInnovations, mF, mStatePred, mCovStatePred] = KalmanFilter(vy, mZ, mGG, mT, mHH, mHG, va, mP);
% Standardised innovations
vStandInn = vInnovations ./ sqrt(dsigma2 * mF);
subplot(2,1,1),
plot(1:cn, vStandInn); title('Standardised Innovations');
subplot(2,1,2),
autocorr(vStandInn(2:end), 20); 
% State one-step-ahead predictions
clf;
vstd_pred = sqrt(dsigma2 * mCovStatePred);
subplot(2,1,1),
plot(1:cn, var1); title('True signal');
subplot(2,1,2),
plot(1:cn, mStatePred, '-b', 1:cn, mStatePred+2*vstd_pred, '.r', 1:cn, mStatePred-2*vstd_pred, '.r'); 
legend('state prediction', 'upper cl', 'lower cl');

clf;
[vRealTimeLevel, vVarRTLevel] = KFRealTime(vy, mZ, mGG, mT, mHH, mHG, va, mP);
plot(1:cn, var1, 'r', 1:cn, vRealTimeLevel, 'b');
legend('True signal', 'Real time estimate');

plot(1:cn, vy, 'r', ...
     1:cn, vRealTimeLevel, 'b', ...
     1:cn, vRealTimeLevel+1.96 * sqrt(dsigma2 * vVarRTLevel), '.g',...
     1:cn, vRealTimeLevel-1.96 * sqrt(dsigma2 * vVarRTLevel), '.g');
 title('Real time estimates');
 
cn2 = 50; 
[vRealTimeWeights, ] = KFRealTime([zeros(cn2,1); 1; zeros(cn2,1)], mZ, mGG, mT, mHH, mHG, va, mP);
bar(-cn2:cn2, vRealTimeWeights), title('AR(1) signal extraction weights');
